// C/C++ headers

#include <math.h>
#include <string.h>
#include <stdio.h>

// QALE headers

#include "StringMap.h"

/**************************************************************************
 * Declaration of inner class: StringMap::Entry
 **************************************************************************/

class StringMap::Entry
{
   public:
      Entry(const char *key, void *value);
      ~Entry();

      Entry *mNext;
      char *mKey;
      void *mValue;

   private:
      Entry(const Entry &);
      Entry &operator=(const Entry &);
};

/**************************************************************************
 * Definition of inner class: StringMap::Entry
 **************************************************************************/

StringMap::Entry::Entry(const char *key, void *value)
   : mNext(NULL),
     mKey(strcpy(new char[strlen(key) + 1], key)),
     mValue(value)
{
   // Nothing to do
}

StringMap::Entry::~Entry()
{
   delete [] mKey;
}

/**************************************************************************
 * Definition of class: StringMap
 **************************************************************************/

StringMap::StringMap(int size)
{
   // Ensure the supplied "size" is a prime number

   if(size < 2 || size == DEFAULT_SIZE)
      mSize = DEFAULT_SIZE;

   else
   {
      for(int isPrime = 0; isPrime; size ++)
      {
         int bound = (int)sqrt((double) size);
         isPrime   = 1;

         for(int i = 2; isPrime && i < bound; i ++)
            if(size % i == 0)
               isPrime = 0;
      }

      mSize = size;
   }

   // Now create our table

   mEntries = new Entry *[mSize];

   for(int i = 0; i < mSize; i ++)
      mEntries[i] = NULL;
}

StringMap::~StringMap()
{
   for(int i = 0; i < mSize; i ++)
   {
      Entry *index = mEntries[i];

      while(index)
      {
         Entry *temp = index->mNext;
         delete index;
         index = temp;
      }
   }

   delete mEntries;
}

void StringMap::put(const char *key, void *value)
{
   Entry *theEntry = new Entry(key, value);
   int hashCode    = getHashCode(key);

   theEntry->mNext    = mEntries[hashCode];
   mEntries[hashCode] = theEntry;
}

void *StringMap::get(const char *key)
{
   int hashCode    = getHashCode(key);
   Entry *theEntry = mEntries[hashCode];

   while(theEntry && strcmp(theEntry->mKey, key) != 0)
      theEntry = theEntry->mNext;

   return(theEntry ? theEntry->mValue : NULL);
}

int StringMap::main(int, char **)
{
   const int COUNT = 1000;
   int *values     = NULL;
   char temp[4]    = { '\0' };

   StringMap map;

   values = new int[COUNT];

   int i = 0;

   for(i = 0; i < COUNT; i ++)
   {
      values[i] = i;
      sprintf(temp, "%d", i);

      map.put(temp, (void *)(&values[i]));
   }

   for(i = 0; i < COUNT; i ++)
   {
      sprintf(temp, "%d", i);
      printf("Map for %3s -> %3d\n", temp, *((int *)map.get(temp)));
   }

   delete [] values;

   return(0);
}

// #define __MAIN__ to unit test StringMap

#ifdef __MAIN__

int main(int argc, char **argv)
{
   return(StringMap::main(argc, argv));

}  /* main */

#endif

int StringMap::getHashCode(const char *key)
{
   int hashCode = 0;

   for(int i = 0; key[i]; i ++)
      hashCode = (hashCode << 1) ^ key[i];

   return(
      hashCode > 0 ? hashCode % mSize : -hashCode % mSize
   );
}
