#ifndef _WIDGET_H_
#   define _WIDGET_H_

#include <iostream>
using std::ostream;

class Node;
class Alignment;
class Define;
class Component;
class ComponentSupplier;
class Visitor;
class DefineVisitor;
class PropertyVisitor;
class AttributeVisitor;

class Widget
{
   public:
      Widget(
         const char *widgetName,
         const Node *width,
         const Node *height,
         const Node *x,
         const Node *y,
         const Alignment *align,
         int level
      );

      ~Widget();

      void setNDefines(int count);
      void setDefine(int index, Define *theDefine);

      void setNChildren(int count);
      void setChild(int index, Widget *child);

      Widget *getParent();
      const char *getName();

      Widget *getChild(const char *name);

      const Node *getDefine(const char *name);

      void acquireComponent(
         const char *lineage,
         ComponentSupplier *supplier
      );

      Component *getComponent();

      void setQualifiedName(const char *qualifiedName);
      const char *getQualifiedName() const;

      void resolveDefines(
         DefineVisitor &
      );

      void resolveProperties(
         PropertyVisitor &
      );

      void resolveAttributes(
         AttributeVisitor &
      );

      void setSize();
      void setLocation();

      friend ostream &operator<<(ostream &, const Widget &);

   private:
      Widget(const Widget &);
      Widget &operator=(const Widget &);

      void handleVisitor(Visitor &);
      void setParent(Widget *);

      Widget *mParent;
      const char *mName;
      const char *mQualifiedName;
      const Node *mWidth;
      const Node *mHeight;
      const Node *mX;
      const Node *mY;
      const Alignment *mAlign;

      int mNDefines;
      Define **mDefines;

      int mNChildren;
      Widget **mChildren;

      int mLevel;

      Component *mComponent;
};

#endif
